// String encoding conversion library
// Written by Pawel Tatera
// Copyright (c) 2014, 2015 TinyIT
// All rights reserved
//
// For Linux (C99-way) of selecting locales see manual page of setlocale().
// For MS Windows see "Code Page Identifiers" section in SDK.

#ifndef TCONV_H
#define TCONV_H

#include "tconv_so.h"

#ifndef NULL
#define NULL	0L
#endif

#ifdef __cplusplus

#ifndef wchar_t
#include <cwchar>
#endif

class DECLSPEC tconv {
  public:

  // NOTE: error=-1 would end up with different value on 32 and 64 bit CPUs.
  enum { ok=0, error=0xffffffff };

  // Constructor, also stores initial conversion locales for LC_CTYPE.
  tconv(const char* = NULL, const char * = NULL);

  // No destructor decalration required.

  // Used to set internal encoding after when tconv got instantiated.
  void set_ienc(const char* = NULL);
  void set_oenc(const char* = NULL);

  // Length of a multi-byte string in wchar_t characters, without
  // the NULL-terminating byte.
  unsigned int wcs_len(const char *, unsigned char = 0);
 
  // Length of a wide-character string in bytes, after when it would be
  // converted into the multi-byte string, without the NULL-terminating byte.
  unsigned int mbs_len(const wchar_t *, unsigned char = 0);

  // Used to quickly convert from one character set to another.
  // Same memory location can be used as data input and output.
  // Third parameter tells how many characters to convert.
  unsigned int convert(char*, const char *, unsigned int = 0);

  // Two supporting functions to do the conversions from multi-byte to wchar
  // and vice versa. Third parameter tells how many characters to convert.
  unsigned int mb_wc(wchar_t *, const char * *, unsigned int = 0); 
  unsigned int wc_mb(char *, const wchar_t * *, unsigned int = 0); 

  private:
  // The locale_in_use bits. This is a small optimization approach to not
  // call setlocale repeateadly.
  unsigned char iSet : 1;
  unsigned char oSet : 1;

  // Character encoding of input and output strings.
  const char * iEncoding;
  const char * oEncoding;

#ifdef _WIN32
  // Used to catch UTFs
  int cpIn, cpOut;
  int catch_multibyte(const char *enc) const;
#endif // _WIN32

};

extern "C"
{

// Declare C++ code haveing C-names here.

#endif // __cplusplus

// Declare native C code.
// Note that native C does not support the default parameters!
// Part used in tconv-c.cpp

#ifndef wchar_t
#include <wchar.h>
#endif

// status codes (enum?)
#define tconvOk		0
#define tconvError	0xffffffff

// Type declaration, importatn!
typedef struct tconv 	tconv;

// Constructor
DECLSPEC tconv * tconv_init(const char *, const char *);

// Destructor
DECLSPEC void tconv_free(tconv * const);

// Sets up input string encoding.
DECLSPEC void tconv_set_ienc(tconv *, const char * const);

// Sets up output string encoding.
DECLSPEC void tconv_set_oenc(tconv *, const char * const);

// Length in wchar_t-s of the char* pointed string.
DECLSPEC unsigned int tconv_wcslen(tconv *, const char * const,
  unsigned char); 

// Length in multi-byte characters of a *wchar_t pointed string.
DECLSPEC unsigned int tconv_mbslen(tconv *, const wchar_t * const,
  unsigned char); 

// Converter
DECLSPEC unsigned int tconv_convert(tconv *tc, char *destP,
  const char *const srcP, const unsigned int max);

// MB to WC converter
DECLSPEC unsigned int tconv_mbwc(tconv *, wchar_t *, const char * *,
  unsigned int);

// WC to MB converter
DECLSPEC unsigned int tconv_wcmb(tconv *, char *, const wchar_t * *,
  unsigned int); 

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif // TCONV_H

